//
// Created by #Suyghur, on 2021/06/09.
//



#include <jni.h>
#include <jtool.h>
#include <rsa/rsa_utils.h>
#include "include/logger.h"
#include "aes/aes_utils.h"

static void GetAesKey(JNIEnv *env, jobject thiz) {
    uint8_t *aes_key = AesUtils::GetKey();
    uint8_t *iv = AesUtils::GetIv(aes_key);
    LOGD("%s", aes_key);
    LOGD("%s", iv);
    LOGD("------ invoke free() ------ ");
    free(aes_key);
    free(iv);
//    free((void *) aes_key);
//    free((void *) iv);
    aes_key = nullptr;
    iv = nullptr;
    LOGD("%s", aes_key);
    LOGD("%s", iv);
}

static jstring AesEncrypt(JNIEnv *env, jobject thiz, jstring src) {
    const char *src_ = env->GetStringUTFChars(src, JNI_FALSE);
    char *raw = AesUtils::Encrypt(src_);
    LOGD("%s", raw);
    env->ReleaseStringUTFChars(src, src_);

    jstring result = JTool::GetJString(env, raw);
    free(raw);

    return result;
}

static jstring AesDecrypt(JNIEnv *env, jobject thiz, jstring raw, jstring key) {
    const char *raw_ = env->GetStringUTFChars(raw, JNI_FALSE);
    const auto *key_ = reinterpret_cast<const uint8_t *>(env->GetStringUTFChars(key, JNI_FALSE));
    char *src = AesUtils::Decrypt(raw_, key_);
    env->ReleaseStringUTFChars(raw, raw_);

    jstring result = JTool::GetJString(env, src);
    free(src);
    return result;
}


static jstring RsaEncrypt(JNIEnv *env, jobject thiz, jstring src) {
    const char *src_ = env->GetStringUTFChars(src, JNI_FALSE);
    std::string key = "MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCAVatPhPaZtxBkLBZDnOLf7no6VfnjBRMXhpOnWmkkDX4zurB/OKisYDFnR0UUVnKn717+absfLlDk9HZWBcTMznsUju9fuXlu3Elr8HeI3en7E0KIQzxkY1GT1+qtZ+tCjWuyMmUw4vWpO8/MRU3f6nc4io3w+7N+dP24BIo0ZwIDAQAB";
    std::string raw = RsaUtils::EncryptByPublicKey(key, src_);
//    LOGD("base64 raw : %s", raw.c_str());
    env->ReleaseStringUTFChars(src, src_);
    jstring result = JTool::GetJString(env, raw.c_str());
    return result;
}

static jstring RsaDecrypt(JNIEnv *env, jobject thiz, jstring raw) {
    const char *raw_ = env->GetStringUTFChars(raw, JNI_FALSE);
    std::string key = "MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCAVatPhPaZtxBkLBZDnOLf7no6VfnjBRMXhpOnWmkkDX4zurB/OKisYDFnR0UUVnKn717+absfLlDk9HZWBcTMznsUju9fuXlu3Elr8HeI3en7E0KIQzxkY1GT1+qtZ+tCjWuyMmUw4vWpO8/MRU3f6nc4io3w+7N+dP24BIo0ZwIDAQAB";
    std::string src = RsaUtils::DecryptByPublicKey(key, raw_);
    env->ReleaseStringUTFChars(raw, raw_);
    jstring result = JTool::GetJString(env, src.c_str());
    return result;
}


static JNINativeMethod gMethod[] = {
        {"getAesKey",  "()V",                                                      (void *) GetAesKey},
        {"aesEncrypt", "(Ljava/lang/String;)Ljava/lang/String;",                   (void *) AesEncrypt},
        {"aesDecrypt", "(Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;", (void *) AesDecrypt},
        {"rsaEncrypt", "(Ljava/lang/String;)Ljava/lang/String;",                   (void *) RsaEncrypt},
        {"rsaDecrypt", "(Ljava/lang/String;)Ljava/lang/String;",                   (void *) RsaDecrypt}
};

extern "C" JNIEXPORT jint JNICALL JNI_OnLoad(JavaVM *vm, void *reserved) {
    JNIEnv *env = nullptr;
    if (vm->GetEnv((void **) &env, JNI_VERSION_1_6) != JNI_OK) {
        return JNI_ERR;
    }
    jclass clz = env->FindClass("cn/yyxx/columbus/core/SdkDrive");
    if (env->RegisterNatives(clz, gMethod, sizeof(gMethod) / sizeof(gMethod[0])) < 0) {
        return JNI_ERR;
    }
    return JNI_VERSION_1_6;
}